#!/bin/sh -e
BLOCK_SIZE=4096
round_4K_up ()
{
	MOD=$(echo "$1 % $BLOCK_SIZE"|bc)
	if [ $MOD -gt 0 ]
	then 
		echo "( $1 / $BLOCK_SIZE ) +1" | bc
	else
		echo "( $1 / $BLOCK_SIZE )" | bc
	fi
	
}
round_4K_down ()
{
	echo "$1 / $BLOCK_SIZE" | bc
}
setup_local()
{
	get_range ()
	{
		local START=$(round_4K_down $1)
		local END=$(round_4K_up $2)
		local IFILE=$3
		local OFILE=$4
		local COUNT=$(expr $END - $START)
		
		dd if="$IFILE" of="$OFILE" bs=$BLOCK_SIZE \
			skip=$START seek=$START count=$COUNT conv=notrunc \
			2>/dev/null
	}
	get_size ()
	{
		local FILE="$1"
		stat -c "%s" "$FILE"
	}
}
setup_remote()
{
	get_range ()
	{
		local START=$(round_4K_down $1)
		local END=$(round_4K_up $2)
		local IFILE="$3"
		local OFILE="$4"
		local BYTE_START=$(expr $START '*' $BLOCK_SIZE)
		local BYTE_END=$(expr $END '*' $BLOCK_SIZE)
		
		curl -s -S --range $BYTE_START-$BYTE_END "$IFILE" | \
			dd bs=$BLOCK_SIZE seek=$START conv=notrunc of="$OFILE" \
				2>/dev/null
	}
	get_size ()
	{
		local FILE="$1"
		curl -s -S --head "$FILE" | \
			sed -n -e 's/Content-Length: \([0-9]*\).*/\1/p'
	}
}

IFILE=$1 #../AV_WINXP_PC_USB_SR0201.zip
OFILE=$(tempfile) #AV_WINXP_PC_USB_SR0201-zerofill.zip
trap "rm \"$OFILE\"" EXIT
ENTRY=$2 #USB/WLAGS51B.sys

case "$IFILE" in
http:*|ftp:*|https:*)
	setup_remote;;
*)
	setup_local;;
esac

SIZE=$(get_size "$IFILE")
DIR_START=$(expr $SIZE - $BLOCK_SIZE)

echo central-directory start: $DIR_START end: $SIZE 
get_range $DIR_START $SIZE "$IFILE" "$OFILE"
NUMBERS="$(zipinfo -v "$OFILE" "$ENTRY" | sed -n \
  -e '/Central directory entry/,/MS-DOS file attributes/{' \
  -e 's/ offset of local header from start of archive: *\([0-9]*\) .*/\1/p' \
  -e 's/ compressed size: *\([0-9]*\) .*/\1/p' \
  -e '}')"

ENTRY_START=$(echo "$NUMBERS" | sed -n -e '1p')
ENTRY_END=$(echo "$NUMBERS" | sed -n -e "2s/$/+$ENTRY_START+256/p"|bc)

echo "$ENTRY" start: "$ENTRY_START" end: "$ENTRY_END"
get_range "$ENTRY_START" "$ENTRY_END" "$IFILE" "$OFILE"
ENTRY_TARGET=$(basename "$ENTRY")
unzip -qq -c "$OFILE" "$ENTRY" > "$ENTRY_TARGET"
echo Saved "$ENTRY" as "$ENTRY_TARGET"
